#!/usr/bin/env python
import re
import sys
import shutil
import os.path
import subprocess
import sysconfig

import reindent
import untabify


SRCDIR = sysconfig.get_config_var('srcdir')


def n_files_str(count):
    """Return 'N file(s)' with the proper plurality on 'file'."""
    return "{} file{}".format(count, "s" if count != 1 else "")


def status(message, modal=False, info=None):
    """Decorator to output status info to stdout."""
    def decorated_fxn(fxn):
        def call_fxn(*args, **kwargs):
            sys.stdout.write(message + ' ... ')
            sys.stdout.flush()
            result = fxn(*args, **kwargs)
            if not modal and not info:
                print "done"
            elif info:
                print info(result)
            else:
                print "yes" if result else "NO"
            return result
        return call_fxn
    return decorated_fxn


def mq_patches_applied():
    """Check if there are any applied MQ patches."""
    cmd = 'hg qapplied'
    st = subprocess.Popen(cmd.split(),
                          stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE)
    try:
        bstdout, _ = st.communicate()
        return st.returncode == 0 and bstdout
    finally:
        st.stdout.close()
        st.stderr.close()


@status("Getting the list of files that have been added/changed",
        info=lambda x: n_files_str(len(x)))
def changed_files():
    """Get the list of changed or added files from the VCS."""
    if os.path.isdir(os.path.join(SRCDIR, '.hg')):
        vcs = 'hg'
        cmd = 'hg status --added --modified --no-status'
        if mq_patches_applied():
            cmd += ' --rev qparent'
    elif os.path.isdir('.svn'):
        vcs = 'svn'
        cmd = 'svn status --quiet --non-interactive --ignore-externals'
    else:
        sys.exit('need a checkout to get modified files')

    st = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE)
    try:
        st.wait()
        if vcs == 'hg':
            return [x.decode().rstrip() for x in st.stdout]
        else:
            output = (x.decode().rstrip().rsplit(None, 1)[-1]
                      for x in st.stdout if x[0] in 'AM')
        return set(path for path in output if os.path.isfile(path))
    finally:
        st.stdout.close()


def report_modified_files(file_paths):
    count = len(file_paths)
    if count == 0:
        return n_files_str(count)
    else:
        lines = ["{}:".format(n_files_str(count))]
        for path in file_paths:
            lines.append("  {}".format(path))
        return "\n".join(lines)


@status("Fixing whitespace", info=report_modified_files)
def normalize_whitespace(file_paths):
    """Make sure that the whitespace for .py files have been normalized."""
    reindent.makebackup = False  # No need to create backups.
    fixed = []
    for path in (x for x in file_paths if x.endswith('.py')):
        if reindent.check(os.path.join(SRCDIR, path)):
            fixed.append(path)
    return fixed


@status("Fixing C file whitespace", info=report_modified_files)
def normalize_c_whitespace(file_paths):
    """Report if any C files """
    fixed = []
    for path in file_paths:
        abspath = os.path.join(SRCDIR, path)
        with open(abspath, 'r') as f:
            if '\t' not in f.read():
                continue
        untabify.process(abspath, 8, verbose=False)
        fixed.append(path)
    return fixed


ws_re = re.compile(br'\s+(\r?\n)$')

@status("Fixing docs whitespace", info=report_modified_files)
def normalize_docs_whitespace(file_paths):
    fixed = []
    for path in file_paths:
        abspath = os.path.join(SRCDIR, path)
        try:
            with open(abspath, 'rb') as f:
                lines = f.readlines()
            new_lines = [ws_re.sub(br'\1', line) for line in lines]
            if new_lines != lines:
                shutil.copyfile(abspath, abspath + '.bak')
                with open(abspath, 'wb') as f:
                    f.writelines(new_lines)
                fixed.append(path)
        except Exception as err:
            print 'Cannot fix %s: %s' % (path, err)
    return fixed


@status("Docs modified", modal=True)
def docs_modified(file_paths):
    """Report if any file in the Doc directory has been changed."""
    return bool(file_paths)


@status("Misc/ACKS updated", modal=True)
def credit_given(file_paths):
    """Check if Misc/ACKS has been changed."""
    return os.path.join('Misc', 'ACKS') in file_paths


@status("Misc/NEWS updated", modal=True)
def reported_news(file_paths):
    """Check if Misc/NEWS has been changed."""
    return os.path.join('Misc', 'NEWS') in file_paths


def main():
    file_paths = changed_files()
    python_files = [fn for fn in file_paths if fn.endswith('.py')]
    c_files = [fn for fn in file_paths if fn.endswith(('.c', '.h'))]
    doc_files = [fn for fn in file_paths if fn.startswith('Doc')]
    misc_files = {os.path.join('Misc', 'ACKS'), os.path.join('Misc', 'NEWS')}\
            & set(file_paths)
    # PEP 8 whitespace rules enforcement.
    normalize_whitespace(python_files)
    # C rules enforcement.
    normalize_c_whitespace(c_files)
    # Doc whitespace enforcement.
    normalize_docs_whitespace(doc_files)
    # Docs updated.
    docs_modified(doc_files)
    # Misc/ACKS changed.
    credit_given(misc_files)
    # Misc/NEWS changed.
    reported_news(misc_files)

    # Test suite run and passed.
    if python_files or c_files:
        end = " and check for refleaks?" if c_files else "?"
        print
        print "Did you run the test suite" + end


if __name__ == '__main__':
    main()



# Scan an Apple header file, generating a Python file of generator calls.

import sys
from bgenlocations import TOOLBOXDIR, BGENDIR
sys.path.append(BGENDIR)

from scantools import Scanner

def main():
    input = "QuickDraw.h"
    output = "qdgen.py"
    defsoutput = TOOLBOXDIR + "QuickDraw.py"
    scanner = MyScanner(input, output, defsoutput)
    scanner.scan()
    scanner.close()

    # Grmpf. Universal Headers have Text-stuff in a different include file...
    input = "QuickDrawText.h"
    output = "@qdgentext.py"
    defsoutput = "@QuickDrawText.py"
    have_extra = 0
    try:
        scanner = MyScanner(input, output, defsoutput)
        scanner.scan()
        scanner.close()
        have_extra = 1
    except IOError:
        pass
    if have_extra:
        print "=== Copying QuickDrawText stuff into main files... ==="
        ifp = open("@qdgentext.py")
        ofp = open("qdgen.py", "a")
        ofp.write(ifp.read())
        ifp.close()
        ofp.close()
        ifp = open("@QuickDrawText.py")
        ofp = open(TOOLBOXDIR + "QuickDraw.py", "a")
        ofp.write(ifp.read())
        ifp.close()
        ofp.close()

    print "=== Testing definitions output code ==="
    execfile(defsoutput, {}, {})
    print "=== Done scanning and generating, now importing the generated code... ==="
    import qdsupport
    print "=== Done.  It's up to you to compile it now! ==="

class MyScanner(Scanner):

    def destination(self, type, name, arglist):
        classname = "Function"
        listname = "functions"
        if arglist:
            t, n, m = arglist[0]
            if t in ('GrafPtr', 'CGrafPtr') and m == 'InMode':
                classname = "Method"
                listname = "gr_methods"
            elif t == 'BitMapPtr' and m == 'InMode':
                classname = "Method"
                listname = "bm_methods"
##                      elif t == "PolyHandle" and m == "InMode":
##                              classname = "Method"
##                              listname = "p_methods"
##                      elif t == "RgnHandle" and m == "InMode":
##                              classname = "Method"
##                              listname = "r_methods"
        return classname, listname


    def writeinitialdefs(self):
        self.defsfile.write("""
def FOUR_CHAR_CODE(x): return x
normal                                          = 0
bold                                            = 1
italic                                          = 2
underline                                       = 4
outline                                         = 8
shadow                                          = 0x10
condense                                        = 0x20
extend                                          = 0x40
""")

    def makeblacklistnames(self):
        return [
                'InitGraf',
                'StuffHex',
                'StdLine',
                'StdComment',
                'StdGetPic',
                'OpenPort',
                'InitPort',
                'ClosePort',
                'OpenCPort',
                'InitCPort',
                'CloseCPort',
                'BitMapToRegionGlue',
                'StdOpcode',    # XXXX Missing from library...
                # The following are for non-macos use:
                'LockPortBits',
                'UnlockPortBits',
                'UpdatePort',
                'GetPortNativeWindow',
                'GetNativeWindowPort',
                'NativeRegionToMacRegion',
                'MacRegionToNativeRegion',
                'GetPortHWND',
                'GetHWNDPort',
                'GetPICTFromDIB',

                'HandleToRgn', # Funny signature

                # Need Cm, which we don't want to drag in just yet
                'OpenCursorComponent',
                'CloseCursorComponent',
                'SetCursorComponent',
                'CursorComponentChanged',
                'CursorComponentSetData',
                ]

    def makeblacklisttypes(self):
        return [
                "QDRegionBitsRef", # Should do this, but too lazy now.
                'CIconHandle', # Obsolete
                'CQDProcs',
                'CQDProcsPtr',
                'CSpecArray',
                'ColorComplementProcPtr',
                'ColorComplementUPP',
                'ColorSearchProcPtr',
                'ColorSearchUPP',
                'ConstPatternParam',
                'DeviceLoopDrawingProcPtr',
                'DeviceLoopFlags',
                'GrafVerb',
                'OpenCPicParams_ptr',
                'Ptr',
                'QDProcs',
                'ReqListRec',
                'void_ptr',
                'CustomXFerProcPtr',
                ]

    def makerepairinstructions(self):
        return [
                ([('void_ptr', 'textBuf', 'InMode'),
                  ('short', 'firstByte', 'InMode'),
                  ('short', 'byteCount', 'InMode')],
                 [('TextThingie', '*', '*'), ('*', '*', '*'), ('*', '*', '*')]),

                # GetPen and SetPt use a point-pointer as output-only:
                ('GetPen', [('Point', '*', 'OutMode')], [('*', '*', 'OutMode')]),
                ('SetPt', [('Point', '*', 'OutMode')], [('*', '*', 'OutMode')]),

                # All others use it as input/output:
                ([('Point', '*', 'OutMode')],
                 [('*', '*', 'InOutMode')]),

                 # InsetRect, OffsetRect
                 ([('Rect', 'r', 'OutMode'),
                        ('short', 'dh', 'InMode'),
                        ('short', 'dv', 'InMode')],
                  [('Rect', 'r', 'InOutMode'),
                        ('short', 'dh', 'InMode'),
                        ('short', 'dv', 'InMode')]),

                 # MapRect
                 ([('Rect', 'r', 'OutMode'),
                        ('Rect_ptr', 'srcRect', 'InMode'),
                        ('Rect_ptr', 'dstRect', 'InMode')],
                  [('Rect', 'r', 'InOutMode'),
                        ('Rect_ptr', 'srcRect', 'InMode'),
                        ('Rect_ptr', 'dstRect', 'InMode')]),

                 # CopyBits and friends
                 ([('RgnHandle', 'maskRgn', 'InMode')],
                  [('OptRgnHandle', 'maskRgn', 'InMode')]),

                 ('QDFlushPortBuffer',
                  [('RgnHandle', '*', 'InMode')],
                  [('OptRgnHandle', '*', 'InMode')]),

                 # Accessors with reference argument also returned.
                 ([('Rect_ptr', 'GetPortBounds', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('RGBColor_ptr', 'GetPortForeColor', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('RGBColor_ptr', 'GetPortBackColor', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('RGBColor_ptr', 'GetPortOpColor', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('RGBColor_ptr', 'GetPortHiliteColor', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Point_ptr', 'GetPortPenSize', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Point_ptr', 'GetPortPenLocation', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Rect_ptr', 'GetPixBounds', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('BitMap_ptr', 'GetQDGlobalsScreenBits', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Cursor_ptr', 'GetQDGlobalsArrow', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Rect_ptr', 'GetRegionBounds', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Pattern_ptr', '*', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Point_ptr', 'QDLocalToGlobalPoint', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Rect_ptr', 'QDLocalToGlobalRect', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Point_ptr', 'QDGlobalToLocalPoint', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                 ([('Rect_ptr', 'QDGlobalToLocalRect', 'ReturnMode')],
                  [('void', '*', 'ReturnMode')]),

                ]

if __name__ == "__main__":
    main()



"""Suite Special Events: Commands for mounting Disk Copy images
Level 1, version 1

Generated from Macintosh HD:Hulpprogramma's:Disk Copy
AETE/AEUT resource version 1/0, language 0, script 0
"""

import aetools
import MacOS

_code = 'ddsk'

class Special_Events_Events:

    _argmap_mount = {
            'access_mode' : 'Acss',
            'checksum_verification' : 'VChk',
            'signature_verification' : 'VSig',
            'RAM_caching' : 'Cach',
    }

    def mount(self, _object, _attributes={}, **_arguments):
        """mount: Mounts an Disk Copy image as a disk volume
        Required argument: a reference to the disk image to be mounted
        Keyword argument access_mode: the access mode for mounted volume (default is "any", i.e. best possible)
        Keyword argument checksum_verification: Verify the checksum before mounting?
        Keyword argument signature_verification: Verify the DigiSignŽ signature before mounting?
        Keyword argument RAM_caching: Cache the disk image in RAM? (if omitted, don't cache)
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: a reference to mounted disk
        """
        _code = 'ddsk'
        _subcode = 'Moun'

        aetools.keysubst(_arguments, self._argmap_mount)
        _arguments['----'] = _object

        aetools.enumsubst(_arguments, 'Acss', _Enum_Acss)
        aetools.enumsubst(_arguments, 'VChk', _Enum_bool)
        aetools.enumsubst(_arguments, 'VSig', _Enum_bool)
        aetools.enumsubst(_arguments, 'Cach', _Enum_bool)

        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_execute_DiskScript = {
            'checksum_verification' : 'VChk',
            'signature_verification' : 'VSig',
    }

    def execute_DiskScript(self, _object, _attributes={}, **_arguments):
        """execute DiskScript: Executes a Disk Copy-specific DiskScript
        Required argument: a reference to the DiskScript to execute
        Keyword argument checksum_verification: Should checksums be verified when mounting images referenced in the DiskScript?
        Keyword argument signature_verification: Should the DigiSignŽ signature of the DiskScript and the images it references be verified?
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ddsk'
        _subcode = 'XEQd'

        aetools.keysubst(_arguments, self._argmap_execute_DiskScript)
        _arguments['----'] = _object

        aetools.enumsubst(_arguments, 'VChk', _Enum_bool)
        aetools.enumsubst(_arguments, 'VSig', _Enum_bool)

        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def unmount(self, _object, _attributes={}, **_arguments):
        """unmount: Unmount and eject (if necessary) a volume
        Required argument: a reference to disk to be unmounted (and ejected)
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ddsk'
        _subcode = 'Umnt'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_create = {
            'saving_as' : 'SvAs',
            'logical_blocks' : 'Blks',
            'zeroing' : 'Zero',
            'leave_image_mounted' : 'Moun',
            'filesystem' : 'Fsys',
    }

    def create(self, _object, _attributes={}, **_arguments):
        """create: Create a new Disk Copy document
        Required argument: the name of the volume to create
        Keyword argument saving_as: the disk image to be created
        Keyword argument logical_blocks: the number of logical blocks
        Keyword argument zeroing: Should all blocks on the disk be set to zero?
        Keyword argument leave_image_mounted: Should the image be mounted after it is created?
        Keyword argument filesystem: file system to use (Mac OS Standard/compatible, Mac OS Enhanced)
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: a reference to newly created disk image (or newly mounted disk)
        """
        _code = 'ddsk'
        _subcode = 'Crea'

        aetools.keysubst(_arguments, self._argmap_create)
        _arguments['----'] = _object

        aetools.enumsubst(_arguments, 'SvAs', _Enum_fss_)
        aetools.enumsubst(_arguments, 'Blks', _Enum_long)
        aetools.enumsubst(_arguments, 'Zero', _Enum_bool)
        aetools.enumsubst(_arguments, 'Moun', _Enum_bool)
        aetools.enumsubst(_arguments, 'Fsys', _Enum_Fsys)

        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def verify_checksum(self, _object, _attributes={}, **_arguments):
        """verify checksum: Verify the checksum of a Disk Copy 4.2 or a Disk Copy 6.0 read-only document
        Required argument: the disk image to be verified
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: the result of the checksum verification
        """
        _code = 'ddsk'
        _subcode = 'Vcrc'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    def verify_signature(self, _object, _attributes={}, **_arguments):
        """verify signature: Verify the DigiSignŽ signature for a Disk Copy document
        Required argument: the disk image to be verified
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: Is the DigiSignŽ signature valid?
        """
        _code = 'ddsk'
        _subcode = 'Vsig'

        if _arguments: raise TypeError, 'No optional args expected'
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_sign_image = {
            'using_signer' : 'Sinr',
    }

    def sign_image(self, _object, _attributes={}, **_arguments):
        """sign image: Add a DigiSignŽ signature to a Disk Copy document
        Required argument: the disk image to be signed
        Keyword argument using_signer: a reference to signer file to use
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ddsk'
        _subcode = 'Asig'

        aetools.keysubst(_arguments, self._argmap_sign_image)
        _arguments['----'] = _object

        aetools.enumsubst(_arguments, 'Sinr', _Enum_alis)

        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_create_a_floppy_from = {
            'signature_verification' : 'VSig',
            'erase_confirmation' : 'Cfrm',
            'make_multiple_floppies' : 'Mult',
    }

    def create_a_floppy_from(self, _object, _attributes={}, **_arguments):
        """create a floppy from: create a floppy disk from a Disk Copy document
        Required argument: the disk image to make a floppy from
        Keyword argument signature_verification: Should the DigiSignŽ signature be verified before creating a floppy disk?
        Keyword argument erase_confirmation: Should the user be asked to confirm the erasure of the previous contents of floppy disks?
        Keyword argument make_multiple_floppies: Should the user be prompted to create multiple floppy disks?
        Keyword argument _attributes: AppleEvent attribute dictionary
        """
        _code = 'ddsk'
        _subcode = 'Bfpy'

        aetools.keysubst(_arguments, self._argmap_create_a_floppy_from)
        _arguments['----'] = _object

        aetools.enumsubst(_arguments, 'VSig', _Enum_bool)
        aetools.enumsubst(_arguments, 'Cfrm', _Enum_bool)
        aetools.enumsubst(_arguments, 'Mult', _Enum_bool)

        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_check_image = {
            'details' : 'ChDe',
    }

    def check_image(self, _object, _attributes={}, **_arguments):
        """check image: Check the disk imageÕs internal data structures for any inconsistencies.  Works on NDIF, Disk Copy 4.2, DARTŽ, or DiskSet images.
        Required argument: the disk image to be verified
        Keyword argument details: Should the disk image details be displayed?
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: a record containing a boolean (true/false) value if the image passes consistency tests, and the numbers of warnings and errors
        """
        _code = 'ddsk'
        _subcode = 'Chek'

        aetools.keysubst(_arguments, self._argmap_check_image)
        _arguments['----'] = _object

        aetools.enumsubst(_arguments, 'ChDe', _Enum_bool)

        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_segment_image = {
            'segment_count' : 'SGCT',
            'segment_size' : 'SGSZ',
            'segment_name' : 'SGNM',
            'image_ID' : 'SGID',
    }

    def segment_image(self, _object, _attributes={}, **_arguments):
        """segment image: Segment a NDIF R/W or R/O image into smaller pieces
        Required argument: the disk image to be segmented
        Keyword argument segment_count: the number of image segments to create
        Keyword argument segment_size: the size of image segments (in blocks) to create
        Keyword argument segment_name: the root name for each image segment file
        Keyword argument image_ID: string used to generate a unique image ID to group the segments
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: a list of references to the image segments created
        """
        _code = 'ddsk'
        _subcode = 'SGMT'

        aetools.keysubst(_arguments, self._argmap_segment_image)
        _arguments['----'] = _object


        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']

    _argmap_create_SMI = {
            'source_images' : 'SMI1',
            'launching_application' : 'SMI2',
            'launching_document' : 'SMI3',
            'version_string' : 'SMI4',
            'checksum_verification' : 'VChk',
            'signature_verification' : 'VSig',
            'image_signing' : 'SImg',
    }

    def create_SMI(self, _object, _attributes={}, **_arguments):
        """create SMI: Creates a self-mounting image (SMI) from a list of NDIF disk images
        Required argument: the self-mounting image to create
        Keyword argument source_images: a list of references to sources images
        Keyword argument launching_application: the path to an application to launch
        Keyword argument launching_document: the path to a document to open
        Keyword argument version_string: sets the 'vers' 1 resource of the self-mounting image
        Keyword argument checksum_verification: Should the checksum of the source images be verified before creating the SMI?
        Keyword argument signature_verification: Should the DigiSignŽ signature of the source images be verified before creating the SMI?
        Keyword argument image_signing: Should the SMI be given a digital signature when it is created?
        Keyword argument _attributes: AppleEvent attribute dictionary
        Returns: a reference to the self-mounting image created
        """
        _code = 'ddsk'
        _subcode = 'MSMI'

        aetools.keysubst(_arguments, self._argmap_create_SMI)
        _arguments['----'] = _object

        aetools.enumsubst(_arguments, 'VChk', _Enum_bool)
        aetools.enumsubst(_arguments, 'VSig', _Enum_bool)
        aetools.enumsubst(_arguments, 'SImg', _Enum_bool)

        _reply, _arguments, _attributes = self.send(_code, _subcode,
                        _arguments, _attributes)
        if _arguments.has_key('errn'):
            raise aetools.Error, aetools.decodeerror(_arguments)
        # XXXX Optionally decode result
        if _arguments.has_key('----'):
            return _arguments['----']


class Verify_Checksum_reply_record(aetools.ComponentItem):
    """Verify Checksum reply record -  """
    want = 'Rcrc'
class validity(aetools.NProperty):
    """validity - true if checksum is valid """
    which = 'Vlid'
    want = 'bool'
class expected_checksum(aetools.NProperty):
    """expected checksum - checksum value stored in the image header (in hexadecimal) """
    which = 'crcE'
    want = 'TEXT'
class calculated_checksum(aetools.NProperty):
    """calculated checksum - checksum value actually calculated (in hexadecimal) """
    which = 'crcA'
    want = 'TEXT'

class Check_Image_reply_record(aetools.ComponentItem):
    """Check Image reply record -  """
    want = 'Rchk'
class consistency(aetools.NProperty):
    """consistency - Does the image pass consistency checks? """
    which = 'Rch1'
    want = 'bool'
class error_count(aetools.NProperty):
    """error count - the number of errors recorded """
    which = 'Rch2'
    want = 'long'
class warning_count(aetools.NProperty):
    """warning count - the number of warnings recorded """
    which = 'Rch3'
    want = 'long'
Verify_Checksum_reply_record._propdict = {
        'validity' : validity,
        'expected_checksum' : expected_checksum,
        'calculated_checksum' : calculated_checksum,
}
Verify_Checksum_reply_record._elemdict = {
}
Check_Image_reply_record._propdict = {
        'consistency' : consistency,
        'error_count' : error_count,
        'warning_count' : warning_count,
}
Check_Image_reply_record._elemdict = {
}
_Enum_Acss = {
        'read_and_write' : 'RdWr',      # read/write access
        'read_only' : 'Rdxx',   # read-only access
        'any' : 'Anyx', # best possible access
}

_Enum_Fsys = {
        'Mac_OS_Standard' : 'Fhfs',     # classic HFS file system
        'compatible_Mac_OS_Extended' : 'Fhf+',  # new HFS+ file system
}

_Enum_alis = None # XXXX enum alis not found!!
_Enum_fss_ = None # XXXX enum fss  not found!!
_Enum_long = None # XXXX enum long not found!!
_Enum_bool = None # XXXX enum bool not found!!

#
# Indices of types declared in this module
#
_classdeclarations = {
        'Rchk' : Check_Image_reply_record,
        'Rcrc' : Verify_Checksum_reply_record,
}

_propdeclarations = {
        'crcE' : expected_checksum,
        'Rch2' : error_count,
        'crcA' : calculated_checksum,
        'Rch3' : warning_count,
        'Vlid' : validity,
        'Rch1' : consistency,
}

_compdeclarations = {
}

_enumdeclarations = {
        'Acss' : _Enum_Acss,
        'Fsys' : _Enum_Fsys,
}



# This script generates a Python interface for an Apple Macintosh Manager.
# It uses the "bgen" package to generate C code.
# The function specifications are generated by scanning the mamager's header file,
# using the "scantools" package (customized for this particular manager).

import string

# Declarations that change for each manager
MACHEADERFILE = 'QuickDraw.h'           # The Apple header file
MODNAME = '_Qd'                         # The name of the module
OBJECTNAME = 'Graf'                     # The basic name of the objects used here

# The following is *usually* unchanged but may still require tuning
MODPREFIX = 'Qd'                        # The prefix for module-wide routines
OBJECTTYPE = OBJECTNAME + 'Ptr'         # The C type used to represent them
OBJECTPREFIX = MODPREFIX + 'Obj'        # The prefix for object methods
INPUTFILE = string.lower(MODPREFIX) + 'gen.py' # The file generated by the scanner
EXTRAFILE = string.lower(MODPREFIX) + 'edit.py' # A similar file but hand-made
OUTPUTFILE = MODNAME + "module.c"       # The file generated by this program

from macsupport import *

# Create the type objects

class TextThingieClass(FixedInputBufferType):
    def getargsCheck(self, name):
        Output("/* Fool compiler warnings */")
        Output("%s__in_len__ = %s__in_len__;", name, name)

    def declareSize(self, name):
        Output("int %s__in_len__;", name)

TextThingie = TextThingieClass(None)

# These are temporary!
RgnHandle = OpaqueByValueType("RgnHandle", "ResObj")
OptRgnHandle = OpaqueByValueType("RgnHandle", "OptResObj")
PicHandle = OpaqueByValueType("PicHandle", "ResObj")
PolyHandle = OpaqueByValueType("PolyHandle", "ResObj")
PixMapHandle = OpaqueByValueType("PixMapHandle", "ResObj")
PixPatHandle = OpaqueByValueType("PixPatHandle", "ResObj")
PatHandle = OpaqueByValueType("PatHandle", "ResObj")
CursHandle = OpaqueByValueType("CursHandle", "ResObj")
CCrsrHandle = OpaqueByValueType("CCrsrHandle", "ResObj")
CIconHandle = OpaqueByValueType("CIconHandle", "ResObj")
CTabHandle = OpaqueByValueType("CTabHandle", "ResObj")
ITabHandle = OpaqueByValueType("ITabHandle", "ResObj")
GDHandle = OpaqueByValueType("GDHandle", "ResObj")
CGrafPtr = OpaqueByValueType("CGrafPtr", "GrafObj")
GrafPtr = OpaqueByValueType("GrafPtr", "GrafObj")
BitMap_ptr = OpaqueByValueType("BitMapPtr", "BMObj")
const_BitMap_ptr = OpaqueByValueType("const BitMap *", "BMObj")
BitMap = OpaqueType("BitMap", "BMObj_NewCopied", "BUG")
RGBColor = OpaqueType('RGBColor', 'QdRGB')
RGBColor_ptr = RGBColor
FontInfo = OpaqueType('FontInfo', 'QdFI')
Component = OpaqueByValueType('Component', 'CmpObj')
ComponentInstance = OpaqueByValueType('ComponentInstance', 'CmpInstObj')

Cursor = StructOutputBufferType('Cursor')
Cursor_ptr = StructInputBufferType('Cursor')
Pattern = StructOutputBufferType('Pattern')
Pattern_ptr = StructInputBufferType('Pattern')
PenState = StructOutputBufferType('PenState')
PenState_ptr = StructInputBufferType('PenState')
TruncCode = Type("TruncCode", "h")

includestuff = includestuff + """
#include <Carbon/Carbon.h>

#ifdef USE_TOOLBOX_OBJECT_GLUE
extern PyObject *_GrafObj_New(GrafPtr);
extern int _GrafObj_Convert(PyObject *, GrafPtr *);
extern PyObject *_BMObj_New(BitMapPtr);
extern int _BMObj_Convert(PyObject *, BitMapPtr *);
extern PyObject *_QdRGB_New(RGBColorPtr);
extern int _QdRGB_Convert(PyObject *, RGBColorPtr);

#define GrafObj_New _GrafObj_New
#define GrafObj_Convert _GrafObj_Convert
#define BMObj_New _BMObj_New
#define BMObj_Convert _BMObj_Convert
#define QdRGB_New _QdRGB_New
#define QdRGB_Convert _QdRGB_Convert
#endif

static PyObject *BMObj_NewCopied(BitMapPtr);

/*
** Parse/generate RGB records
*/
PyObject *QdRGB_New(RGBColorPtr itself)
{

        return Py_BuildValue("lll", (long)itself->red, (long)itself->green, (long)itself->blue);
}

int QdRGB_Convert(PyObject *v, RGBColorPtr p_itself)
{
        long red, green, blue;

        if( !PyArg_ParseTuple(v, "lll", &red, &green, &blue) )
                return 0;
        p_itself->red = (unsigned short)red;
        p_itself->green = (unsigned short)green;
        p_itself->blue = (unsigned short)blue;
        return 1;
}

/*
** Generate FontInfo records
*/
static
PyObject *QdFI_New(FontInfo *itself)
{

        return Py_BuildValue("hhhh", itself->ascent, itself->descent,
                        itself->widMax, itself->leading);
}
"""

finalstuff = finalstuff + """
/* Like BMObj_New, but the original bitmap data structure is copied (and
** released when the object is released)
*/
PyObject *BMObj_NewCopied(BitMapPtr itself)
{
        BitMapObject *it;
        BitMapPtr itself_copy;

        if ((itself_copy=(BitMapPtr)malloc(sizeof(BitMap))) == NULL)
                return PyErr_NoMemory();
        *itself_copy = *itself;
        it = (BitMapObject *)BMObj_New(itself_copy);
        it->referred_bitmap = itself_copy;
        return (PyObject *)it;
}

"""

variablestuff = ""

initstuff = initstuff + """
        PyMac_INIT_TOOLBOX_OBJECT_NEW(BitMapPtr, BMObj_New);
        PyMac_INIT_TOOLBOX_OBJECT_CONVERT(BitMapPtr, BMObj_Convert);
        PyMac_INIT_TOOLBOX_OBJECT_NEW(GrafPtr, GrafObj_New);
        PyMac_INIT_TOOLBOX_OBJECT_CONVERT(GrafPtr, GrafObj_Convert);
        PyMac_INIT_TOOLBOX_OBJECT_NEW(RGBColorPtr, QdRGB_New);
        PyMac_INIT_TOOLBOX_OBJECT_CONVERT(RGBColor, QdRGB_Convert);
"""

## not yet...
##
##class Region_ObjectDefinition(GlobalObjectDefinition):
##      def outputCheckNewArg(self):
##              Output("if (itself == NULL) return PyMac_Error(resNotFound);")
##      def outputFreeIt(self, itselfname):
##              Output("DisposeRegion(%s);", itselfname)
##
##class Polygon_ObjectDefinition(GlobalObjectDefinition):
##      def outputCheckNewArg(self):
##              Output("if (itself == NULL) return PyMac_Error(resNotFound);")
##      def outputFreeIt(self, itselfname):
##              Output("KillPoly(%s);", itselfname)

class MyGRObjectDefinition(PEP253Mixin, GlobalObjectDefinition):
    getsetlist = [
            ('visRgn',
            """RgnHandle h=NewRgn(); /* XXXX wrong dispose routine */
            return Py_BuildValue("O&", ResObj_New, (Handle)GetPortVisibleRegion(self->ob_itself, h));
            """,
            None,
            "Convenience attribute: return a copy of the visible region"
            ), (
            'clipRgn',
            """RgnHandle h=NewRgn(); /* XXXX wrong dispose routine */
            return Py_BuildValue("O&", ResObj_New, (Handle)GetPortClipRegion(self->ob_itself, h));
            """,
            None,
            "Convenience attribute: return a copy of the clipping region"
            )]
    def outputCheckNewArg(self):
        Output("if (itself == NULL) return PyMac_Error(resNotFound);")
    def outputCheckConvertArg(self):
        Output("#if 1")
        OutLbrace()
        Output("WindowRef win;")
        OutLbrace("if (WinObj_Convert(v, &win) && v)")
        Output("*p_itself = (GrafPtr)GetWindowPort(win);")
        Output("return 1;")
        OutRbrace()
        Output("PyErr_Clear();")
        OutRbrace()
        Output("#else")
        OutLbrace("if (DlgObj_Check(v))")
        Output("DialogRef dlg = (DialogRef)((GrafPortObject *)v)->ob_itself;")
        Output("*p_itself = (GrafPtr)GetWindowPort(GetDialogWindow(dlg));")
        Output("return 1;")
        OutRbrace()
        OutLbrace("if (WinObj_Check(v))")
        Output("WindowRef win = (WindowRef)((GrafPortObject *)v)->ob_itself;")
        Output("*p_itself = (GrafPtr)GetWindowPort(win);")
        Output("return 1;")
        OutRbrace()
        Output("#endif")

class MyBMObjectDefinition(PEP253Mixin, GlobalObjectDefinition):
    getsetlist = [
    (
    'baseAddr',
    'return PyInt_FromLong((long)self->ob_itself->baseAddr);',
    None,
    None
    ), (
    'rowBytes',
    'return PyInt_FromLong((long)self->ob_itself->rowBytes);',
    None,
    None
    ), (
    'bounds',
    'return Py_BuildValue("O&", PyMac_BuildRect, &self->ob_itself->bounds);',
    None,
    None
    ), (
    'bitmap_data',
    'return PyString_FromStringAndSize((char *)self->ob_itself, sizeof(BitMap));',
    None,
    None
    ), (
    'pixmap_data',
    'return PyString_FromStringAndSize((char *)self->ob_itself, sizeof(PixMap));',
    None,
    None
    )]
    def outputCheckNewArg(self):
        Output("if (itself == NULL) return PyMac_Error(resNotFound);")
    def outputStructMembers(self):
        # We need to more items: a pointer to privately allocated data
        # and a python object we're referring to.
        Output("%s ob_itself;", self.itselftype)
        Output("PyObject *referred_object;")
        Output("BitMap *referred_bitmap;")
    def outputInitStructMembers(self):
        Output("it->ob_itself = %sitself;", self.argref)
        Output("it->referred_object = NULL;")
        Output("it->referred_bitmap = NULL;")
    def outputCleanupStructMembers(self):
        Output("Py_XDECREF(self->referred_object);")
        Output("if (self->referred_bitmap) free(self->referred_bitmap);")

# Create the generator groups and link them
module = MacModule(MODNAME, MODPREFIX, includestuff, finalstuff, initstuff, variablestuff)
##r_object = Region_ObjectDefinition('Region', 'QdRgn', 'RgnHandle')
##po_object = Polygon_ObjectDefinition('Polygon', 'QdPgn', 'PolyHandle')
##module.addobject(r_object)
##module.addobject(po_object)
gr_object = MyGRObjectDefinition("GrafPort", "GrafObj", "GrafPtr")
module.addobject(gr_object)
bm_object = MyBMObjectDefinition("BitMap", "BMObj", "BitMapPtr")
module.addobject(bm_object)


# Create the generator classes used to populate the lists
Function = OSErrWeakLinkFunctionGenerator
Method = OSErrWeakLinkMethodGenerator

# Create and populate the lists
functions = []
gr_methods = []
bm_methods = []
#methods = []
execfile(INPUTFILE)
execfile(EXTRAFILE)

# add the populated lists to the generator groups
# (in a different wordl the scan program would generate this)
for f in functions: module.add(f)
for f in gr_methods: gr_object.add(f)
for f in bm_methods: bm_object.add(f)

# Manual generator: get data out of a bitmap
getdata_body = """
int from, length;
char *cp;

if ( !PyArg_ParseTuple(_args, "ii", &from, &length) )
        return NULL;
cp = _self->ob_itself->baseAddr+from;
_res = PyString_FromStringAndSize(cp, length);
return _res;
"""
f = ManualGenerator("getdata", getdata_body)
f.docstring = lambda: """(int start, int size) -> string. Return bytes from the bitmap"""
bm_object.add(f)

# Manual generator: store data in a bitmap
putdata_body = """
int from, length;
char *cp, *icp;

if ( !PyArg_ParseTuple(_args, "is#", &from, &icp, &length) )
        return NULL;
cp = _self->ob_itself->baseAddr+from;
memcpy(cp, icp, length);
Py_INCREF(Py_None);
_res = Py_None;
return _res;
"""
f = ManualGenerator("putdata", putdata_body)
f.docstring = lambda: """(int start, string data). Store bytes into the bitmap"""
bm_object.add(f)

#
# We manually generate a routine to create a BitMap from python data.
#
BitMap_body = """
BitMap *ptr;
PyObject *source;
Rect bounds;
int rowbytes;
char *data;

if ( !PyArg_ParseTuple(_args, "O!iO&", &PyString_Type, &source, &rowbytes, PyMac_GetRect,
                &bounds) )
        return NULL;
data = PyString_AsString(source);
if ((ptr=(BitMap *)malloc(sizeof(BitMap))) == NULL )
        return PyErr_NoMemory();
ptr->baseAddr = (Ptr)data;
ptr->rowBytes = rowbytes;
ptr->bounds = bounds;
if ( (_res = BMObj_New(ptr)) == NULL ) {
        free(ptr);
        return NULL;
}
((BitMapObject *)_res)->referred_object = source;
Py_INCREF(source);
((BitMapObject *)_res)->referred_bitmap = ptr;
return _res;
"""

f = ManualGenerator("BitMap", BitMap_body)
f.docstring = lambda: """Take (string, int, Rect) argument and create BitMap"""
module.add(f)

#
# And again, for turning a correctly-formatted structure into the object
#
RawBitMap_body = """
BitMap *ptr;
PyObject *source;

if ( !PyArg_ParseTuple(_args, "O!", &PyString_Type, &source) )
        return NULL;
if ( PyString_Size(source) != sizeof(BitMap) && PyString_Size(source) != sizeof(PixMap) ) {
        PyErr_Format(PyExc_TypeError,
                "Argument size was %d, should be %d (sizeof BitMap) or %d (sizeof PixMap)",
                PyString_Size(source), sizeof(BitMap), sizeof(PixMap));
        return NULL;
}
ptr = (BitMapPtr)PyString_AsString(source);
if ( (_res = BMObj_New(ptr)) == NULL ) {
        return NULL;
}
((BitMapObject *)_res)->referred_object = source;
Py_INCREF(source);
return _res;
"""

f = ManualGenerator("RawBitMap", RawBitMap_body)
f.docstring = lambda: """Take string BitMap and turn into BitMap object"""
module.add(f)

# generate output (open the output file as late as possible)
SetOutputFileName(OUTPUTFILE)
module.generate()
SetOutputFile() # Close it



"""Ttk Theme Selector v2.

This is an improvement from the other theme selector (themes_combo.py)
since now you can notice theme changes in Ttk Combobox, Ttk Frame,
Ttk Label and Ttk Button.
"""
import Tkinter
import ttk

class App(ttk.Frame):
    def __init__(self):
        ttk.Frame.__init__(self, borderwidth=3)

        self.style = ttk.Style()

        # XXX Ideally I wouldn't want to create a Tkinter.IntVar to make
        #     it works with Checkbutton variable option.
        self.theme_autochange = Tkinter.IntVar(self, 0)
        self._setup_widgets()

    def _change_theme(self):
        self.style.theme_use(self.themes_combo.get())

    def _theme_sel_changed(self, widget):
        if self.theme_autochange.get():
            self._change_theme()

    def _setup_widgets(self):
        themes_lbl = ttk.Label(self, text="Themes")

        themes = self.style.theme_names()
        self.themes_combo = ttk.Combobox(self, values=themes, state="readonly")
        self.themes_combo.set(themes[0])
        self.themes_combo.bind("<<ComboboxSelected>>", self._theme_sel_changed)

        change_btn = ttk.Button(self, text='Change Theme',
            command=self._change_theme)

        theme_change_checkbtn = ttk.Checkbutton(self,
            text="Change themes when combobox item is activated",
            variable=self.theme_autochange)

        themes_lbl.grid(ipadx=6, sticky="w")
        self.themes_combo.grid(row=0, column=1, padx=6, sticky="ew")
        change_btn.grid(row=0, column=2, padx=6, sticky="e")
        theme_change_checkbtn.grid(row=1, columnspan=3, sticky="w", pady=6)

        top = self.winfo_toplevel()
        top.rowconfigure(0, weight=1)
        top.columnconfigure(0, weight=1)
        self.columnconfigure(1, weight=1)
        self.grid(row=0, column=0, sticky="nsew", columnspan=3, rowspan=2)


def main():
    app = App()
    app.master.title("Theme Selector")
    app.mainloop()

if __name__ == "__main__":
    main()



"""
A demo that reads in an RSS XML document and emits an HTML file containing
a list of the individual items in the feed.
"""

import sys
import codecs

from xml.sax import make_parser, handler

# --- Templates

top = """\
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
  <title>%s</title>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
</head>

<body>
<h1>%s</h1>
"""

bottom = """
</ul>

<hr>
<address>
Converted to HTML by rss2html.py.
</address>

</body>
</html>
"""

# --- The ContentHandler

class RSSHandler(handler.ContentHandler):

    def __init__(self, out=sys.stdout):
        handler.ContentHandler.__init__(self)
        self._out = codecs.getwriter('utf-8')(out)

        self._text = ""
        self._parent = None
        self._list_started = False
        self._title = None
        self._link = None
        self._descr = ""

    # ContentHandler methods

    def startElement(self, name, attrs):
        if name == "channel" or name == "image" or name == "item":
            self._parent = name

        self._text = ""

    def endElement(self, name):
        if self._parent == "channel":
            if name == "title":
                self._out.write(top % (self._text, self._text))
            elif name == "description":
                self._out.write("<p>%s</p>\n" % self._text)

        elif self._parent == "item":
            if name == "title":
                self._title = self._text
            elif name == "link":
                self._link = self._text
            elif name == "description":
                self._descr = self._text
            elif name == "item":
                if not self._list_started:
                    self._out.write("<ul>\n")
                    self._list_started = True

                self._out.write('  <li><a href="%s">%s</a> %s\n' %
                                (self._link, self._title, self._descr))

                self._title = None
                self._link = None
                self._descr = ""

        if name == "rss":
            self._out.write(bottom)

    def characters(self, content):
        self._text = self._text + content

# --- Main program

if __name__ == '__main__':
    parser = make_parser()
    parser.setContentHandler(RSSHandler())
    parser.parse(sys.argv[1])



#!/usr/bin/env python

import unittest
import sys
import os
import _testcapi
from test import test_support
from subprocess import Popen, PIPE

# Skip this test if the _tkinter module wasn't built.
_tkinter = test_support.import_module('_tkinter')

from Tkinter import Tcl
from _tkinter import TclError

tcl_version = _tkinter.TCL_VERSION.split('.')
try:
    for i in range(len(tcl_version)):
        tcl_version[i] = int(tcl_version[i])
except ValueError:
    pass
tcl_version = tuple(tcl_version)


class TkinterTest(unittest.TestCase):

    def testFlattenLen(self):
        # flatten(<object with no length>)
        self.assertRaises(TypeError, _tkinter._flatten, True)


class TclTest(unittest.TestCase):

    def setUp(self):
        self.interp = Tcl()

    def testEval(self):
        tcl = self.interp
        tcl.eval('set a 1')
        self.assertEqual(tcl.eval('set a'),'1')

    def testEvalException(self):
        tcl = self.interp
        self.assertRaises(TclError,tcl.eval,'set a')

    def testEvalException2(self):
        tcl = self.interp
        self.assertRaises(TclError,tcl.eval,'this is wrong')

    def testCall(self):
        tcl = self.interp
        tcl.call('set','a','1')
        self.assertEqual(tcl.call('set','a'),'1')

    def testCallException(self):
        tcl = self.interp
        self.assertRaises(TclError,tcl.call,'set','a')

    def testCallException2(self):
        tcl = self.interp
        self.assertRaises(TclError,tcl.call,'this','is','wrong')

    def testSetVar(self):
        tcl = self.interp
        tcl.setvar('a','1')
        self.assertEqual(tcl.eval('set a'),'1')

    def testSetVarArray(self):
        tcl = self.interp
        tcl.setvar('a(1)','1')
        self.assertEqual(tcl.eval('set a(1)'),'1')

    def testGetVar(self):
        tcl = self.interp
        tcl.eval('set a 1')
        self.assertEqual(tcl.getvar('a'),'1')

    def testGetVarArray(self):
        tcl = self.interp
        tcl.eval('set a(1) 1')
        self.assertEqual(tcl.getvar('a(1)'),'1')

    def testGetVarException(self):
        tcl = self.interp
        self.assertRaises(TclError,tcl.getvar,'a')

    def testGetVarArrayException(self):
        tcl = self.interp
        self.assertRaises(TclError,tcl.getvar,'a(1)')

    def testUnsetVar(self):
        tcl = self.interp
        tcl.setvar('a',1)
        self.assertEqual(tcl.eval('info exists a'),'1')
        tcl.unsetvar('a')
        self.assertEqual(tcl.eval('info exists a'),'0')

    def testUnsetVarArray(self):
        tcl = self.interp
        tcl.setvar('a(1)',1)
        tcl.setvar('a(2)',2)
        self.assertEqual(tcl.eval('info exists a(1)'),'1')
        self.assertEqual(tcl.eval('info exists a(2)'),'1')
        tcl.unsetvar('a(1)')
        self.assertEqual(tcl.eval('info exists a(1)'),'0')
        self.assertEqual(tcl.eval('info exists a(2)'),'1')

    def testUnsetVarException(self):
        tcl = self.interp
        self.assertRaises(TclError,tcl.unsetvar,'a')

    def testEvalFile(self):
        tcl = self.interp
        filename = "testEvalFile.tcl"
        fd = open(filename,'w')
        script = """set a 1
        set b 2
        set c [ expr $a + $b ]
        """
        fd.write(script)
        fd.close()
        tcl.evalfile(filename)
        os.remove(filename)
        self.assertEqual(tcl.eval('set a'),'1')
        self.assertEqual(tcl.eval('set b'),'2')
        self.assertEqual(tcl.eval('set c'),'3')

    def testEvalFileException(self):
        tcl = self.interp
        filename = "doesnotexists"
        try:
            os.remove(filename)
        except Exception,e:
            pass
        self.assertRaises(TclError,tcl.evalfile,filename)

    def testPackageRequireException(self):
        tcl = self.interp
        self.assertRaises(TclError,tcl.eval,'package require DNE')

    def testLoadWithUNC(self):
        import sys
        if sys.platform != 'win32':
            return

        # Build a UNC path from the regular path.
        # Something like
        #   \\%COMPUTERNAME%\c$\python27\python.exe

        fullname = os.path.abspath(sys.executable)
        if fullname[1] != ':':
            return
        unc_name = r'\\%s\%s$\%s' % (os.environ['COMPUTERNAME'],
                                    fullname[0],
                                    fullname[3:])

        with test_support.EnvironmentVarGuard() as env:
            env.unset("TCL_LIBRARY")
            cmd = '%s -c "import Tkinter; print Tkinter"' % (unc_name,)

            p = Popen(cmd, stdout=PIPE, stderr=PIPE)
            out_data, err_data = p.communicate()

            msg = '\n\n'.join(['"Tkinter.py" not in output',
                               'Command:', cmd,
                               'stdout:', out_data,
                               'stderr:', err_data])

            self.assertIn('Tkinter.py', out_data, msg)

            self.assertEqual(p.wait(), 0, 'Non-zero exit code')


    def test_passing_values(self):
        def passValue(value):
            return self.interp.call('set', '_', value)
        self.assertEqual(passValue(True), True)
        self.assertEqual(passValue(False), False)
        self.assertEqual(passValue('string'), 'string')
        self.assertEqual(passValue('string\u20ac'), 'string\u20ac')
        self.assertEqual(passValue(u'string'), u'string')
        self.assertEqual(passValue(u'string\u20ac'), u'string\u20ac')
        for i in (0, 1, -1, int(2**31-1), int(-2**31)):
            self.assertEqual(passValue(i), i)
        for f in (0.0, 1.0, -1.0, 1//3, 1/3.0,
                  sys.float_info.min, sys.float_info.max,
                  -sys.float_info.min, -sys.float_info.max):
            self.assertEqual(passValue(f), f)
        for f in float('nan'), float('inf'), -float('inf'):
            if f != f: # NaN
                self.assertNotEqual(passValue(f), f)
            else:
                self.assertEqual(passValue(f), f)
        self.assertEqual(passValue((1, '2', (3.4,))), (1, '2', (3.4,)))

    def test_splitlist(self):
        splitlist = self.interp.tk.splitlist
        call = self.interp.tk.call
        self.assertRaises(TypeError, splitlist)
        self.assertRaises(TypeError, splitlist, 'a', 'b')
        self.assertRaises(TypeError, splitlist, 2)
        testcases = [
            ('2', ('2',)),
            ('', ()),
            ('{}', ('',)),
            ('""', ('',)),
            ('a\n b\t\r c\n ', ('a', 'b', 'c')),
            (u'a\n b\t\r c\n ', ('a', 'b', 'c')),
            ('a \xe2\x82\xac', ('a', '\xe2\x82\xac')),
            (u'a \u20ac', ('a', '\xe2\x82\xac')),
            ('a {b c}', ('a', 'b c')),
            (r'a b\ c', ('a', 'b c')),
            (('a', 'b c'), ('a', 'b c')),
            ('a 2', ('a', '2')),
            (('a', 2), ('a', 2)),
            ('a 3.4', ('a', '3.4')),
            (('a', 3.4), ('a', 3.4)),
            ((), ()),
            (call('list', 1, '2', (3.4,)), (1, '2', (3.4,))),
        ]
        if tcl_version >= (8, 5):
            testcases += [
                (call('dict', 'create', 1, u'\u20ac', '\xe2\x82\xac', (3.4,)),
                        (1, u'\u20ac', u'\u20ac', (3.4,))),
            ]
        for arg, res in testcases:
            self.assertEqual(splitlist(arg), res)
        self.assertRaises(TclError, splitlist, '{')

    def test_split(self):
        split = self.interp.tk.split
        call = self.interp.tk.call
        self.assertRaises(TypeError, split)
        self.assertRaises(TypeError, split, 'a', 'b')
        self.assertRaises(TypeError, split, 2)
        testcases = [
            ('2', '2'),
            ('', ''),
            ('{}', ''),
            ('""', ''),
            ('{', '{'),
            ('a\n b\t\r c\n ', ('a', 'b', 'c')),
            (u'a\n b\t\r c\n ', ('a', 'b', 'c')),
            ('a \xe2\x82\xac', ('a', '\xe2\x82\xac')),
            (u'a \u20ac', ('a', '\xe2\x82\xac')),
            ('a {b c}', ('a', ('b', 'c'))),
            (r'a b\ c', ('a', ('b', 'c'))),
            (('a', 'b c'), ('a', ('b', 'c'))),
            (('a', u'b c'), ('a', ('b', 'c'))),
            ('a 2', ('a', '2')),
            (('a', 2), ('a', 2)),
            ('a 3.4', ('a', '3.4')),
            (('a', 3.4), ('a', 3.4)),
            (('a', (2, 3.4)), ('a', (2, 3.4))),
            ((), ()),
            (call('list', 1, '2', (3.4,)), (1, '2', (3.4,))),
        ]
        if tcl_version >= (8, 5):
            testcases += [
                (call('dict', 'create', 12, u'\u20ac', '\xe2\x82\xac', (3.4,)),
                        (12, u'\u20ac', u'\u20ac', (3.4,))),
            ]
        for arg, res in testcases:
            self.assertEqual(split(arg), res)


class BigmemTclTest(unittest.TestCase):

    def setUp(self):
        self.interp = Tcl()

    @unittest.skipUnless(_testcapi.INT_MAX < _testcapi.PY_SSIZE_T_MAX,
                         "needs UINT_MAX < SIZE_MAX")
    @test_support.precisionbigmemtest(size=_testcapi.INT_MAX + 1, memuse=5,
                                      dry_run=False)
    def test_huge_string(self, size):
        value = ' ' * size
        self.assertRaises(OverflowError, self.interp.call, 'set', '_', value)


def test_main():
    test_support.run_unittest(TclTest, TkinterTest, BigmemTclTest)

if __name__ == "__main__":
    test_main()



#!/usr/bin/env python
import sys
import os

from Tkinter import *
from idlelib.Percolator import Percolator
from idlelib.ColorDelegator import ColorDelegator
from idlelib.textView import TextViewer

import turtle
import time

STARTUP = 1
READY = 2
RUNNING = 3
DONE = 4
EVENTDRIVEN = 5

menufont = ("Arial", 12, NORMAL)
btnfont = ("Arial", 12, 'bold')
txtfont = ('Lucida Console', 8, 'normal')

def getExampleEntries():
    cwd = os.getcwd()
    if "turtleDemo.py" not in os.listdir(cwd):
        print "Directory of turtleDemo must be current working directory!"
        print "But in your case this is", cwd
        sys.exit()
    entries1 = [entry for entry in os.listdir(cwd) if
                     entry.startswith("tdemo_") and
                     not entry.endswith(".pyc")]
    entries2 = []
    for entry in entries1:
        if entry.endswith(".py"):
            entries2.append(entry)
        else:
            path = os.path.join(cwd,entry)
            sys.path.append(path)
            subdir = [entry]
            scripts = [script for script in os.listdir(path) if
                            script.startswith("tdemo_") and
                            script.endswith(".py")]
            entries2.append(subdir+scripts)
    return entries2

def showDemoHelp():
    TextViewer(demo.root, "Help on turtleDemo", "demohelp.txt")

def showAboutDemo():
    TextViewer(demo.root, "About turtleDemo", "about_turtledemo.txt")

def showAboutTurtle():
    TextViewer(demo.root, "About the new turtle module", "about_turtle.txt")

class DemoWindow(object):

    def __init__(self, filename=None):   #, root=None):
        self.root = root = turtle._root = Tk()
        root.wm_protocol("WM_DELETE_WINDOW", self._destroy)

        #################
        self.mBar = Frame(root, relief=RAISED, borderwidth=2)
        self.mBar.pack(fill=X)

        self.ExamplesBtn = self.makeLoadDemoMenu()
        self.OptionsBtn = self.makeHelpMenu()
        self.mBar.tk_menuBar(self.ExamplesBtn, self.OptionsBtn) #, QuitBtn)

        root.title('Python turtle-graphics examples')
        #################
        self.left_frame = left_frame = Frame(root)
        self.text_frame = text_frame = Frame(left_frame)
        self.vbar = vbar =Scrollbar(text_frame, name='vbar')
        self.text = text = Text(text_frame,
                                name='text', padx=5, wrap='none',
                                width=45)
        vbar['command'] = text.yview
        vbar.pack(side=LEFT, fill=Y)
        #####################
        self.hbar = hbar =Scrollbar(text_frame, name='hbar', orient=HORIZONTAL)
        hbar['command'] = text.xview
        hbar.pack(side=BOTTOM, fill=X)
        #####################
        text['yscrollcommand'] = vbar.set
        text.config(font=txtfont)
        text.config(xscrollcommand=hbar.set)
        text.pack(side=LEFT, fill=Y, expand=1)
        #####################
        self.output_lbl = Label(left_frame, height= 1,text=" --- ", bg = "#ddf",
                                font = ("Arial", 16, 'normal'))
        self.output_lbl.pack(side=BOTTOM, expand=0, fill=X)
        #####################
        text_frame.pack(side=LEFT, fill=BOTH, expand=0)
        left_frame.pack(side=LEFT, fill=BOTH, expand=0)
        self.graph_frame = g_frame = Frame(root)

        turtle._Screen._root = g_frame
        turtle._Screen._canvas = turtle.ScrolledCanvas(g_frame, 800, 600, 1000, 800)
        #xturtle.Screen._canvas.pack(expand=1, fill="both")
        self.screen = _s_ = turtle.Screen()
        turtle.TurtleScreen.__init__(_s_, _s_._canvas)
        self.scanvas = _s_._canvas
        #xturtle.RawTurtle.canvases = [self.scanvas]
        turtle.RawTurtle.screens = [_s_]

        self.scanvas.pack(side=TOP, fill=BOTH, expand=1)

        self.btn_frame = btn_frame = Frame(g_frame, height=100)
        self.start_btn = Button(btn_frame, text=" START ", font=btnfont, fg = "white",
                                disabledforeground = "#fed", command=self.startDemo)
        self.start_btn.pack(side=LEFT, fill=X, expand=1)
        self.stop_btn = Button(btn_frame, text=" STOP ",  font=btnfont, fg = "white",
                                disabledforeground = "#fed", command = self.stopIt)
        self.stop_btn.pack(side=LEFT, fill=X, expand=1)
        self.clear_btn = Button(btn_frame, text=" CLEAR ",  font=btnfont, fg = "white",
                                disabledforeground = "#fed", command = self.clearCanvas)
        self.clear_btn.pack(side=LEFT, fill=X, expand=1)

        self.btn_frame.pack(side=TOP, fill=BOTH, expand=0)
        self.graph_frame.pack(side=TOP, fill=BOTH, expand=1)

        Percolator(text).insertfilter(ColorDelegator())
        self.dirty = False
        self.exitflag = False
        if filename:
            self.loadfile(filename)
        self.configGUI(NORMAL, DISABLED, DISABLED, DISABLED,
                       "Choose example from menu", "black")
        self.state = STARTUP

    def _destroy(self):
        self.root.destroy()
        sys.exit()

    def configGUI(self, menu, start, stop, clear, txt="", color="blue"):
        self.ExamplesBtn.config(state=menu)

        self.start_btn.config(state=start)
        if start==NORMAL:
            self.start_btn.config(bg="#d00")
        else:
            self.start_btn.config(bg="#fca")

        self.stop_btn.config(state=stop)
        if stop==NORMAL:
            self.stop_btn.config(bg="#d00")
        else:
            self.stop_btn.config(bg="#fca")
        self.clear_btn.config(state=clear)

        self.clear_btn.config(state=clear)
        if clear==NORMAL:
            self.clear_btn.config(bg="#d00")
        else:
            self.clear_btn.config(bg="#fca")

        self.output_lbl.config(text=txt, fg=color)


    def makeLoadDemoMenu(self):
        CmdBtn = Menubutton(self.mBar, text='Examples', underline=0, font=menufont)
        CmdBtn.pack(side=LEFT, padx="2m")
        CmdBtn.menu = Menu(CmdBtn)

        for entry in getExampleEntries():
            def loadexample(x):
                def emit():
                    self.loadfile(x)
                return emit
            if isinstance(entry,str):
                CmdBtn.menu.add_command(label=entry[6:-3], underline=0, font=menufont,
                                        command=loadexample(entry))
            else:
                _dir, entries = entry[0], entry[1:]
                CmdBtn.menu.choices = Menu(CmdBtn.menu)
                for e in entries:
                    CmdBtn.menu.choices.add_command(label=e[6:-3], underline=0, font=menufont,
                              command = loadexample(os.path.join(_dir,e)))

                CmdBtn.menu.add_cascade(label=_dir[6:],
                                        menu = CmdBtn.menu.choices, font=menufont )

        CmdBtn['menu'] = CmdBtn.menu
        return CmdBtn


    def makeHelpMenu(self):
        CmdBtn = Menubutton(self.mBar, text='Help', underline=0, font = menufont)
        CmdBtn.pack(side=LEFT, padx='2m')
        CmdBtn.menu = Menu(CmdBtn)

        CmdBtn.menu.add_command(label='About turtle.py', font=menufont, command=showAboutTurtle)
        CmdBtn.menu.add_command(label='turtleDemo - Help', font=menufont, command=showDemoHelp)
        CmdBtn.menu.add_command(label='About turtleDemo', font=menufont, command=showAboutDemo)

        CmdBtn['menu'] = CmdBtn.menu
        return CmdBtn

    def refreshCanvas(self):
        if not self.dirty: return
        self.screen.clear()
        #self.screen.mode("standard")
        self.dirty=False

    def loadfile(self,filename):
        self.refreshCanvas()
        if os.path.exists(filename) and not os.path.isdir(filename):
            # load and display file text
            f = open(filename,'r')
            chars = f.read()
            f.close()
            self.text.delete("1.0", "end")
            self.text.insert("1.0",chars)
            direc, fname = os.path.split(filename)
            self.root.title(fname[6:-3]+" - a Python turtle graphics example")
            self.module = __import__(fname[:-3])
            reload(self.module)
            self.configGUI(NORMAL, NORMAL, DISABLED, DISABLED,
                           "Press start button", "red")
            self.state = READY

    def startDemo(self):
        self.refreshCanvas()
        self.dirty = True
        turtle.TurtleScreen._RUNNING = True
        self.configGUI(DISABLED, DISABLED, NORMAL, DISABLED,
                       "demo running...", "black")
        self.screen.clear()
        self.screen.mode("standard")
        self.state = RUNNING

        try:
            result = self.module.main()
            if result == "EVENTLOOP":
                self.state = EVENTDRIVEN
            else:
                self.state = DONE
        except turtle.Terminator:
            self.state = DONE
            result = "stopped!"
        if self.state == DONE:
            self.configGUI(NORMAL, NORMAL, DISABLED, NORMAL,
                           result)
        elif self.state == EVENTDRIVEN:
            self.exitflag = True
            self.configGUI(DISABLED, DISABLED, NORMAL, DISABLED,
                           "use mouse/keys or STOP", "red")

    def clearCanvas(self):
        self.refreshCanvas()
        self.scanvas.config(cursor="")
        self.configGUI(NORMAL, NORMAL, DISABLED, DISABLED)

    def stopIt(self):
        if self.exitflag:
            self.clearCanvas()
            self.exitflag = False
            self.configGUI(NORMAL, NORMAL, DISABLED, DISABLED,
                           "STOPPED!", "red")
            turtle.TurtleScreen._RUNNING = False
            #print "stopIT: exitflag = True"
        else:
            turtle.TurtleScreen._RUNNING = False
            #print "stopIt: exitflag = False"

if __name__ == '__main__':
    demo = DemoWindow()
    RUN = True
    while RUN:
        try:
            print "ENTERING mainloop"
            demo.root.mainloop()
        except AttributeError:
            print "CRASH!!!- WAIT A MOMENT!"
            time.sleep(0.3)
            print "GOING ON .."
            demo.refreshCanvas()
##            time.sleep(1)
        except:
            RUN = FALSE



#!/usr/bin/env python

import smtplib

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

# me == my email address
# you == recipient's email address
me = "my@email.com"
you = "your@email.com"

# Create message container - the correct MIME type is multipart/alternative.
msg = MIMEMultipart('alternative')
msg['Subject'] = "Link"
msg['From'] = me
msg['To'] = you

# Create the body of the message (a plain-text and an HTML version).
text = "Hi!\nHow are you?\nHere is the link you wanted:\nhttp://www.python.org"
html = """\
<html>
  <head></head>
  <body>
    <p>Hi!<br>
       How are you?<br>
       Here is the <a href="http://www.python.org">link</a> you wanted.
    </p>
  </body>
</html>
"""

# Record the MIME types of both parts - text/plain and text/html.
part1 = MIMEText(text, 'plain')
part2 = MIMEText(html, 'html')

# Attach parts into message container.
# According to RFC 2046, the last part of a multipart message, in this case
# the HTML message, is best and preferred.
msg.attach(part1)
msg.attach(part2)

# Send the message via local SMTP server.
s = smtplib.SMTP('localhost')
# sendmail function takes 3 arguments: sender's address, recipient's address
# and message to send - here it is sent as one string.
s.sendmail(me, you, msg.as_string())
s.quit()



import os
import unittest
from test import test_support

# Skip this test if _tkinter wasn't built.
test_support.import_module('_tkinter')

this_dir = os.path.dirname(os.path.abspath(__file__))
lib_tk_test = os.path.abspath(os.path.join(this_dir, os.path.pardir,
    'lib-tk', 'test'))

with test_support.DirsOnSysPath(lib_tk_test):
    import runtktests

# Skip test if tk cannot be initialized.
runtktests.check_tk_availability()

import ttk
from _tkinter import TclError

try:
    ttk.Button()
except TclError, msg:
    # assuming ttk is not available
    raise unittest.SkipTest("ttk not available: %s" % msg)

def test_main(enable_gui=False):
    if enable_gui:
        if test_support.use_resources is None:
            test_support.use_resources = ['gui']
        elif 'gui' not in test_support.use_resources:
            test_support.use_resources.append('gui')

    with test_support.DirsOnSysPath(lib_tk_test):
        from test_ttk.support import get_tk_root
        try:
            test_support.run_unittest(
                *runtktests.get_tests(text=False, packages=['test_ttk']))
        finally:
            get_tk_root().destroy()

if __name__ == '__main__':
    test_main(enable_gui=True)



# Import the email modules we'll need
from email.parser import Parser

#  If the e-mail headers are in a file, uncomment this line:
#headers = Parser().parse(open(messagefile, 'r'))

#  Or for parsing headers in a string, use:
headers = Parser().parsestr('From: <user@example.com>\n'
        'To: <someone_else@example.com>\n'
        'Subject: Test message\n'
        '\n'
        'Body would go here\n')

#  Now the header items can be accessed as a dictionary:
print 'To: %s' % headers['to']
print 'From: %s' % headers['from']
print 'Subject: %s' % headers['subject']



"""Demo based on the demo mclist included with tk source distribution."""
import Tkinter
import tkFont
import ttk

tree_columns = ("country", "capital", "currency")
tree_data = [
    ("Argentina",      "Buenos Aires",     "ARS"),
    ("Australia",      "Canberra",         "AUD"),
    ("Brazil",         "Brazilia",         "BRL"),
    ("Canada",         "Ottawa",           "CAD"),
    ("China",          "Beijing",          "CNY"),
    ("France",         "Paris",            "EUR"),
    ("Germany",        "Berlin",           "EUR"),
    ("India",          "New Delhi",        "INR"),
    ("Italy",          "Rome",             "EUR"),
    ("Japan",          "Tokyo",            "JPY"),
    ("Mexico",         "Mexico City",      "MXN"),
    ("Russia",         "Moscow",           "RUB"),
    ("South Africa",   "Pretoria",         "ZAR"),
    ("United Kingdom", "London",           "GBP"),
    ("United States",  "Washington, D.C.", "USD")
    ]

def sortby(tree, col, descending):
    """Sort tree contents when a column is clicked on."""
    # grab values to sort
    data = [(tree.set(child, col), child) for child in tree.get_children('')]

    # reorder data
    data.sort(reverse=descending)
    for indx, item in enumerate(data):
        tree.move(item[1], '', indx)

    # switch the heading so that it will sort in the opposite direction
    tree.heading(col,
        command=lambda col=col: sortby(tree, col, int(not descending)))

class App(object):
    def __init__(self):
        self.tree = None
        self._setup_widgets()
        self._build_tree()

    def _setup_widgets(self):
        msg = ttk.Label(wraplength="4i", justify="left", anchor="n",
            padding=(10, 2, 10, 6),
            text=("Ttk is the new Tk themed widget set. One of the widgets it "
                  "includes is a tree widget, which can be configured to "
                  "display multiple columns of informational data without "
                  "displaying the tree itself. This is a simple way to build "
                  "a listbox that has multiple columns. Clicking on the "
                  "heading for a column will sort the data by that column. "
                  "You can also change the width of the columns by dragging "
                  "the boundary between them."))
        msg.pack(fill='x')

        container = ttk.Frame()
        container.pack(fill='both', expand=True)

        # XXX Sounds like a good support class would be one for constructing
        #     a treeview with scrollbars.
        self.tree = ttk.Treeview(columns=tree_columns, show="headings")
        vsb = ttk.Scrollbar(orient="vertical", command=self.tree.yview)
        hsb = ttk.Scrollbar(orient="horizontal", command=self.tree.xview)
        self.tree.configure(yscrollcommand=vsb.set, xscrollcommand=hsb.set)
        self.tree.grid(column=0, row=0, sticky='nsew', in_=container)
        vsb.grid(column=1, row=0, sticky='ns', in_=container)
        hsb.grid(column=0, row=1, sticky='ew', in_=container)

        container.grid_columnconfigure(0, weight=1)
        container.grid_rowconfigure(0, weight=1)

    def _build_tree(self):
        for col in tree_columns:
            self.tree.heading(col, text=col.title(),
                command=lambda c=col: sortby(self.tree, c, 0))
            # XXX tkFont.Font().measure expected args are incorrect according
            #     to the Tk docs
            self.tree.column(col, width=tkFont.Font().measure(col.title()))

        for item in tree_data:
            self.tree.insert('', 'end', values=item)

            # adjust columns lenghts if necessary
            for indx, val in enumerate(item):
                ilen = tkFont.Font().measure(val)
                if self.tree.column(tree_columns[indx], width=None) < ilen:
                    self.tree.column(tree_columns[indx], width=ilen)

def main():
    root = Tkinter.Tk()
    root.wm_title("Multi-Column List")
    root.wm_iconname("mclist")

    import plastik_theme
    try:
        plastik_theme.install('~/tile-themes/plastik/plastik')
    except Exception:
        import warnings
        warnings.warn("plastik theme being used without images")

    app = App()
    root.mainloop()

if __name__ == "__main__":
    main()



# Scan <Controls.h>, generating ctlgen.py.
import sys
from bgenlocations import TOOLBOXDIR, BGENDIR
sys.path.append(BGENDIR)

from scantools import Scanner

def main():
#       input = "Controls.h" # Universal Headers < 3.3
    input = ["Controls.h", "ControlDefinitions.h"] # Universal Headers >= 3.3
    output = "ctlgen.py"
    defsoutput = TOOLBOXDIR + "Controls.py"
    scanner = MyScanner(input, output, defsoutput)
    scanner.scan()
    scanner.close()
    print "=== Testing definitions output code ==="
    execfile(defsoutput, {}, {})
    print "=== Done scanning and generating, now doing 'import ctlsupport' ==="
    import ctlsupport
    print "=== Done.  It's up to you to compile Ctlmodule.c ==="

class MyScanner(Scanner):

    def destination(self, type, name, arglist):
        classname = "Function"
        listname = "functions"
        if arglist:
            t, n, m = arglist[0]
            if t in ("ControlHandle", "ControlRef") and m == "InMode":
                classname = "Method"
                listname = "methods"
        return classname, listname

    def writeinitialdefs(self):
        self.defsfile.write("def FOUR_CHAR_CODE(x): return x\n")
        self.defsfile.write("from Carbon.TextEdit import *\n")
        self.defsfile.write("from Carbon.QuickDraw import *\n")
        self.defsfile.write("from Carbon.Dragconst import *\n")
        self.defsfile.write("from Carbon.CarbonEvents import *\n")
        self.defsfile.write("from Carbon.Appearance import *\n")
        self.defsfile.write("kDataBrowserItemAnyState = -1\n")
        self.defsfile.write("kControlBevelButtonCenterPopupGlyphTag = -1\n")
        self.defsfile.write("kDataBrowserClientPropertyFlagsMask = 0xFF000000\n")
        self.defsfile.write("\n")

    def makeblacklistnames(self):
        return [
                'FindControlUnderMouse', # Generated manually, returns an existing control, not a new one.
                'DisposeControl', # Generated manually
                'KillControls', # Implied by close of dialog
                'SetCtlAction',
                'TrackControl', # Generated manually
                'HandleControlClick',   # Generated manually
                'SetControlData',       # Generated manually
                'GetControlData',       # Generated manually
                'kControlBevelButtonCenterPopupGlyphTag', # Constant with funny definition
                'kDataBrowserClientPropertyFlagsMask',  # ditto
                'kDataBrowserItemAnyState',   # and ditto
                # The following are unavailable for static 68k (appearance manager)
##                      'GetBevelButtonMenuValue',
##                      'SetBevelButtonMenuValue',
##                      'GetBevelButtonMenuHandle',
##                      'SetBevelButtonTransform',
                'SetBevelButtonGraphicAlignment',
                'SetBevelButtonTextAlignment',
                'SetBevelButtonTextPlacement',
##                      'SetImageWellTransform',
##                      'GetTabContentRect',
##                      'SetTabEnabled',
##                      'SetDisclosureTriangleLastValue',
##                      # Unavailable in CW Pro 3 libraries
##                      'SetUpControlTextColor',
##                      # Unavailable in Jack's CW Pro 5.1 libraries
##                      'GetControlRegion',
##                      'RemoveControlProperty',
##                      'IsValidControlHandle',
##                      'SetControl32BitMinimum',
##                      'GetControl32BitMinimum',
##                      'SetControl32BitMaximum',
##                      'GetControl32BitMaximum',
##                      'SetControl32BitValue',
##                      'GetControl32BitValue',
##                      'SetControlViewSize',
##                      'GetControlViewSize',
                # Generally Bad News
                'GetControlProperty',
                'SetControlProperty',
                'GetControlPropertySize',
                'SendControlMessage', # Parameter changed from long to void* from UH3.3 to UH3.4
                'CreateTabsControl',  # wrote manually
                'GetControlAction',  # too much effort for too little usefulness

                # too lazy for now
                'GetImageWellContentInfo',
                'GetBevelButtonContentInfo',
                # OS8 only
                'GetAuxiliaryControlRecord',
                'SetControlColor',
                ]

    def makeblacklisttypes(self):
        return [
                'ProcPtr',
#                       'ControlActionUPP',
                'Ptr',
                'ControlDefSpec', # Don't know how to do this yet
                'ControlDefSpec_ptr', # ditto
                'Collection', # Ditto
                # not-yet-supported stuff in Universal Headers 3.4:
                'ControlColorUPP',
                'ControlKind',  # XXX easy: 2-tuple containing 2 OSType's
#                       'ControlTabEntry_ptr', # XXX needed for tabs
#                       'ControlButtonContentInfoPtr',
#                       'ControlButtonContentInfo',  # XXX ugh: a union
#                       'ControlButtonContentInfo_ptr',  # XXX ugh: a union
                'ListDefSpec_ptr',  # XXX see _Listmodule.c, tricky but possible
                'DataBrowserItemID_ptr',  # XXX array of UInt32, for BrowserView
                'DataBrowserItemUPP',
                'DataBrowserItemDataRef', # XXX void *
                'DataBrowserCallbacks', # difficult struct
                'DataBrowserCallbacks_ptr',
                'DataBrowserCustomCallbacks',
                'DataBrowserCustomCallbacks_ptr',
##                      'DataBrowserTableViewColumnDesc',
##                      'DataBrowserListViewColumnDesc',
                'CFDataRef',
                'DataBrowserListViewHeaderDesc', # difficult struct
                ]

    def makerepairinstructions(self):
        return [
                ([("void_ptr", "*", "InMode"), ("long", "*", "InMode")],
                 [("InBuffer", "*", "*")]),

                ([("void", "*", "OutMode"), ("long", "*", "InMode"),
                                            ("long", "*", "OutMode")],
                 [("VarVarOutBuffer", "*", "InOutMode")]),

##                      # For TrackControl
##                      ([("ProcPtr", "actionProc", "InMode")],
##                       [("FakeType('(ControlActionUPP)0')", "*", "*")]),
##                      ([("ControlActionUPP", "actionProc", "InMode")],
##                       [("FakeType('(ControlActionUPP)0')", "*", "*")]),

                # For GetControlTitle
                ([('Str255', 'title', 'InMode')],
                 [('Str255', 'title', 'OutMode')]),

                ([("ControlHandle", "*", "OutMode")],
                 [("ExistingControlHandle", "*", "*")]),
                ([("ControlRef", "*", "OutMode")],      # Ditto, for Universal Headers
                 [("ExistingControlHandle", "*", "*")]),

                ([("Rect_ptr", "*", "ReturnMode")], # GetControlBounds
                 [("void", "*", "ReturnMode")]),

                ([("DataBrowserListViewColumnDesc", "*", "OutMode")],
                 [("DataBrowserListViewColumnDesc", "*", "InMode")]),

                ([("ControlButtonContentInfoPtr", 'outContent', "InMode")],
                 [("ControlButtonContentInfoPtr", '*', "OutMode")]),

                ([("ControlButtonContentInfo", '*', "OutMode")],
                 [("ControlButtonContentInfo", '*', "InMode")]),

                ([("ControlActionUPP", 'liveTrackingProc', "InMode")],
                 [("ControlActionUPPNewControl", 'liveTrackingProc', "InMode")]),
                ]

if __name__ == "__main__":
    main()



# Scan an Apple header file, generating a Python file of generator calls.

import sys
from bgenlocations import TOOLBOXDIR, BGENDIR
sys.path.append(BGENDIR)
from scantools import Scanner

LONG = "Lists"
SHORT = "list"
OBJECT = "ListHandle"

def main():
    input = LONG + ".h"
    output = SHORT + "gen.py"
    defsoutput = TOOLBOXDIR + LONG + ".py"
    scanner = MyScanner(input, output, defsoutput)
    scanner.scan()
    scanner.close()
    print "=== Testing definitions output code ==="
    execfile(defsoutput, {}, {})
    print "=== Done scanning and generating, now importing the generated code... ==="
    exec "import " + SHORT + "support"
    print "=== Done.  It's up to you to compile it now! ==="

class MyScanner(Scanner):

    def destination(self, type, name, arglist):
        classname = "Function"
        listname = "functions"
        if arglist:
            t, n, m = arglist[-1]
            # This is non-functional today
            if t in ('ListHandle', 'ListRef') and m == "InMode":
                classname = "Method"
                listname = "methods"
        return classname, listname

    def makeblacklistnames(self):
        return [
                "LDispose",             # Done by removing the object
                "LSearch",              # We don't want to handle procs just yet
                "CreateCustomList",  # done manually
                "SetListDefinitionProc",

                # These have funny argument/return values
                "GetListViewBounds",
                "GetListCellIndent",
                "GetListCellSize",
                "GetListVisibleCells",
                "GetListClickLocation",
                "GetListMouseLocation",
                "GetListDataBounds",
                "SetListLastClick",
                ]

    def makeblacklisttypes(self):
        return [
                "ListClickLoopUPP",  # Too difficult for now
                "ListDefSpecPtr", # later
                ]

    def makerepairinstructions(self):
        return [
                ([('ListBounds_ptr', '*', 'InMode')],
                 [('Rect_ptr', '*', 'InMode')]),

                ([("Cell", "theCell", "OutMode")],
                 [("Cell", "theCell", "InOutMode")]),

                ([("void_ptr", "*", "InMode"), ("short", "*", "InMode")],
                 [("InBufferShortsize", "*", "*")]),

                ([("void", "*", "OutMode"), ("short", "*", "OutMode")],
                 [("VarOutBufferShortsize", "*", "InOutMode")]),

                # SetListCellIndent doesn't have const
                ([("Point", "indent", "OutMode")],
                 [("Point_ptr", "indent", "InMode")]),

                ]

    def writeinitialdefs(self):
        self.defsfile.write("def FOUR_CHAR_CODE(x): return x\n")


if __name__ == "__main__":
    main()



"""This demonstrates good part of the syntax accepted by theme_create.

This is a translation of plastik.tcl to python.
You will need the images used by the plastik theme to test this. The
images (and other tile themes) can be retrived by doing:

$ cvs -z3 -d:pserver:anonymous@tktable.cvs.sourceforge.net:/cvsroot/tktable \
  co tile-themes

To test this module you should do, for example:

import Tkinter
import plastik_theme

root = Tkinter.Tk()
plastik_theme.install(plastik_image_dir)
...

Where plastik_image_dir contains the path to the images directory used by
the plastik theme, something like: tile-themes/plastik/plastik
"""
import os
import glob
import ttk
from Tkinter import PhotoImage

__all__ = ['install']

colors = {
    "frame": "#efefef",
    "disabledfg": "#aaaaaa",
    "selectbg": "#657a9e",
    "selectfg": "#ffffff"
    }

imgs = {}
def _load_imgs(imgdir):
    imgdir = os.path.expanduser(imgdir)
    if not os.path.isdir(imgdir):
        raise Exception("%r is not a directory, can't load images" % imgdir)
    for f in glob.glob("%s/*.gif" % imgdir):
        img = os.path.split(f)[1]
        name = img[:-4]
        imgs[name] = PhotoImage(name, file=f, format="gif89")

def install(imgdir):
    _load_imgs(imgdir)
    style = ttk.Style()
    style.theme_create("plastik", "default", settings={
        ".": {
            "configure":
                {"background": colors['frame'],
                 "troughcolor": colors['frame'],
                 "selectbackground": colors['selectbg'],
                 "selectforeground": colors['selectfg'],
                 "fieldbackground": colors['frame'],
                 "font": "TkDefaultFont",
                 "borderwidth": 1},
            "map": {"foreground": [("disabled", colors['disabledfg'])]}
        },

        "Vertical.TScrollbar": {"layout": [
            ("Vertical.Scrollbar.uparrow", {"side": "top", "sticky": ''}),
            ("Vertical.Scrollbar.downarrow", {"side": "bottom", "sticky": ''}),
            ("Vertical.Scrollbar.uparrow", {"side": "bottom", "sticky": ''}),
            ("Vertical.Scrollbar.trough", {"sticky": "ns", "children":
                [("Vertical.Scrollbar.thumb", {"expand": 1, "unit": 1,
                    "children": [("Vertical.Scrollbar.grip", {"sticky": ''})]
                })]
            })]
        },

        "Horizontal.TScrollbar": {"layout": [
            ("Horizontal.Scrollbar.leftarrow", {"side": "left", "sticky": ''}),
            ("Horizontal.Scrollbar.rightarrow",
                {"side": "right", "sticky": ''}),
            ("Horizontal.Scrollbar.leftarrow",
                {"side": "right", "sticky": ''}),
            ("Horizontal.Scrollbar.trough", {"sticky": "ew", "children":
                [("Horizontal.Scrollbar.thumb", {"expand": 1, "unit": 1,
                    "children": [("Horizontal.Scrollbar.grip", {"sticky": ''})]
                })]
            })]
        },

        "TButton": {
            "configure": {"width": 10, "anchor": "center"},
            "layout": [
                ("Button.button", {"children":
                    [("Button.focus", {"children":
                        [("Button.padding", {"children":
                            [("Button.label", {"side": "left", "expand": 1})]
                        })]
                    })]
                })
            ]
        },

        "Toolbutton": {
            "configure": {"anchor": "center"},
            "layout": [
                ("Toolbutton.border", {"children":
                    [("Toolbutton.button", {"children":
                        [("Toolbutton.padding", {"children":
                            [("Toolbutton.label", {"side":"left", "expand":1})]
                        })]
                    })]
                })
            ]
        },

        "TMenubutton": {"layout": [
            ("Menubutton.button", {"children":
                [("Menubutton.indicator", {"side": "right"}),
                 ("Menubutton.focus", {"children":
                    [("Menubutton.padding", {"children":
                        [("Menubutton.label", {"side": "left", "expand": 1})]
                    })]
                })]
            })]
        },

        "TNotebook": {"configure": {"tabmargins": [0, 2, 0, 0]}},
        "TNotebook.tab": {
            "configure": {"padding": [6, 2, 6, 2], "expand": [0, 0, 2]},
            "map": {"expand": [("selected", [1, 2, 4, 2])]}
        },
        "Treeview": {"configure": {"padding": 0}},

        # elements
        "Button.button": {"element create":
            ("image", 'button-n',
                ("pressed", 'button-p'), ("active", 'button-h'),
                {"border": [4, 10], "padding": 4, "sticky":"ewns"}
            )
        },

        "Toolbutton.button": {"element create":
            ("image", 'tbutton-n',
                ("selected", 'tbutton-p'), ("pressed", 'tbutton-p'),
                ("active", 'tbutton-h'),
                {"border": [4, 9], "padding": 3, "sticky": "news"}
            )
        },

        "Checkbutton.indicator": {"element create":
            ("image", 'check-nu',
                ('active', 'selected', 'check-hc'),
                ('pressed', 'selected', 'check-pc'),
                ('active', 'check-hu'),
                ("selected", 'check-nc'),
                {"sticky": ''}
            )
        },

        "Radiobutton.indicator": {"element create":
            ("image", 'radio-nu',
                ('active', 'selected', 'radio-hc'),
                ('pressed', 'selected', 'radio-pc'),
                ('active', 'radio-hu'), ('selected', 'radio-nc'),
                {"sticky": ''}
            )
        },

        "Horizontal.Scrollbar.thumb": {"element create":
            ("image", 'hsb-n', {"border": 3, "sticky": "ew"})
        },

        "Horizontal.Scrollbar.grip": {"element create": ("image", 'hsb-g')},
        "Horizontal.Scrollbar.trough": {"element create": ("image", 'hsb-t')},
        "Vertical.Scrollbar.thumb": {"element create":
            ("image", 'vsb-n', {"border": 3, "sticky": "ns"})
        },
        "Vertical.Scrollbar.grip": {"element create": ("image", 'vsb-g')},
        "Vertical.Scrollbar.trough": {"element create": ("image", 'vsb-t')},
        "Scrollbar.uparrow": {"element create":
            ("image", 'arrowup-n', ("pressed", 'arrowup-p'), {"sticky": ''})
        },
        "Scrollbar.downarrow": {"element create":
            ("image", 'arrowdown-n',
            ("pressed", 'arrowdown-p'), {'sticky': ''})
        },
        "Scrollbar.leftarrow": {"element create":
            ("image", 'arrowleft-n',
            ("pressed", 'arrowleft-p'), {'sticky': ''})
        },
        "Scrollbar.rightarrow": {"element create":
            ("image", 'arrowright-n', ("pressed", 'arrowright-p'),
            {'sticky': ''})
        },

        "Horizontal.Scale.slider": {"element create":
            ("image", 'hslider-n', {'sticky': ''})
        },
        "Horizontal.Scale.trough": {"element create":
            ("image", 'hslider-t', {'border': 1, 'padding': 0})
        },
        "Vertical.Scale.slider": {"element create":
            ("image", 'vslider-n', {'sticky': ''})
        },
        "Vertical.Scale.trough": {"element create":
            ("image", 'vslider-t', {'border': 1, 'padding': 0})
        },

        "Entry.field": {"element create":
            ("image", 'entry-n',
                ("focus", 'entry-f'),
                {'border': 2, 'padding': [3, 4], 'sticky': 'news'}
            )
        },

        "Labelframe.border": {"element create":
            ("image", 'border', {'border': 4, 'padding': 4, 'sticky': 'news'})
        },

        "Menubutton.button": {"element create":
            ("image", 'combo-r',
                ('active', 'combo-ra'),
                {'sticky': 'news', 'border': [4, 6, 24, 15],
                 'padding': [4, 4, 5]}
            )
        },
        "Menubutton.indicator": {"element create":
            ("image", 'arrow-d', {"sticky": "e", "border": [15, 0, 0, 0]})
        },

        "Combobox.field": {"element create":
            ("image", 'combo-n',
                ('readonly', 'active', 'combo-ra'),
                ('focus', 'active', 'combo-fa'),
                ('active', 'combo-a'), ('!readonly', 'focus', 'combo-f'),
                ('readonly', 'combo-r'),
                {'border': [4, 6, 24, 15], 'padding': [4, 4, 5],
                 'sticky': 'news'}
            )
        },
        "Combobox.downarrow": {"element create":
            ("image", 'arrow-d', {'sticky': 'e', 'border': [15, 0, 0, 0]})
         },

        "Notebook.client": {"element create":
            ("image", 'notebook-c', {'border': 4})
        },
        "Notebook.tab": {"element create":
            ("image", 'notebook-tn',
                ("selected", 'notebook-ts'), ("active", 'notebook-ta'),
                {'padding': [0, 2, 0, 0], 'border': [4, 10, 4, 10]}
            )
        },

        "Progressbar.trough": {"element create":
            ("image", 'hprogress-t', {'border': 2})
        },
        "Horizontal.Progressbar.pbar": {"element create":
            ("image", 'hprogress-b', {'border': [2, 9]})
        },
        "Vertical.Progressbar.pbar": {"element create":
            ("image", 'vprogress-b', {'border': [9, 2]})
        },

        "Treeheading.cell": {"element create":
            ("image", 'tree-n',
                ("pressed", 'tree-p'),
                {'border': [4, 10], 'padding': 4, 'sticky': 'news'}
            )
        }

    })
    style.theme_use("plastik")



# Scan an Apple header file, generating a Python file of generator calls.

import sys
from bgenlocations import TOOLBOXDIR, BGENDIR
sys.path.append(BGENDIR)
from scantools import Scanner

LONG = "QuickTime"
SHORT = "qt"
HEADERFILES= (
#       "Components.h"  -- In Carbon.Cm
        "Movies.h",
        "ImageCompression.h",
        "QuickTimeComponents.h",
#       "ImageCodec.h"  -- seems not too useful, and difficult.
#       "IsochronousDataHandlers.h" -- Is this useful?
        "MediaHandlers.h",
#       "QTML.h", -- Windows only, needs separate module
#       "QuickTimeStreaming.h", -- Difficult
#       "QTStreamingComponents.h", -- Needs QTStreaming
        "QuickTimeMusic.h",
#       "QuickTimeVR.h", -- Not done yet
#       "Sound.h", -- In Carbon.Snd
        )
OBJECTS = ("Movie", "Track", "Media", "UserData", "TimeBase", "MovieController",
        "IdleManager", "SGOutput")

def main():
    input = HEADERFILES
    output = SHORT + "gen.py"
    defsoutput = TOOLBOXDIR + LONG + ".py"
    scanner = MyScanner(input, output, defsoutput)
    scanner.scan()
    scanner.close()
    scanner.gentypetest(SHORT+"typetest.py")
    print "=== Testing definitions output code ==="
    execfile(defsoutput, {}, {})
    print "=== Done scanning and generating, now importing the generated code... ==="
    exec "import " + SHORT + "support"
    print "=== Done.  It's up to you to compile it now! ==="

class MyScanner(Scanner):

    def destination(self, type, name, arglist):
        classname = "Function"
        listname = "functions"
        if arglist:
            t, n, m = arglist[0]
            if t in OBJECTS and m == "InMode":
                classname = "Method"
                listname = t + "_methods"
        return classname, listname

    def writeinitialdefs(self):
        self.defsfile.write("def FOUR_CHAR_CODE(x): return x\n")
        self.defsfile.write("xmlIdentifierUnrecognized = -1\n")
        self.defsfile.write("kControllerMinimum = -0xf777\n")
        self.defsfile.write("notImplementedMusicOSErr      = -2071\n")
        self.defsfile.write("cantSendToSynthesizerOSErr    = -2072\n")
        self.defsfile.write("cantReceiveFromSynthesizerOSErr = -2073\n")
        self.defsfile.write("illegalVoiceAllocationOSErr   = -2074\n")
        self.defsfile.write("illegalPartOSErr              = -2075\n")
        self.defsfile.write("illegalChannelOSErr           = -2076\n")
        self.defsfile.write("illegalKnobOSErr              = -2077\n")
        self.defsfile.write("illegalKnobValueOSErr         = -2078\n")
        self.defsfile.write("illegalInstrumentOSErr        = -2079\n")
        self.defsfile.write("illegalControllerOSErr        = -2080\n")
        self.defsfile.write("midiManagerAbsentOSErr        = -2081\n")
        self.defsfile.write("synthesizerNotRespondingOSErr = -2082\n")
        self.defsfile.write("synthesizerOSErr              = -2083\n")
        self.defsfile.write("illegalNoteChannelOSErr       = -2084\n")
        self.defsfile.write("noteChannelNotAllocatedOSErr  = -2085\n")
        self.defsfile.write("tunePlayerFullOSErr           = -2086\n")
        self.defsfile.write("tuneParseOSErr                = -2087\n")

    def makeblacklistnames(self):
        return [
                "xmlIdentifierUnrecognized", # const with incompatible definition
                "DisposeMovie",         # Done on python-object disposal
                "DisposeMovieTrack",    # ditto
                "DisposeTrackMedia",    # ditto
                "DisposeUserData",              # ditto
#                       "DisposeTimeBase",              # ditto
                "DisposeMovieController", # ditto

                # The following 4 use 'void *' in an uncontrolled way
                # TBD when I've read the manual...
                "GetUserDataItem",
                "SetUserDataItem",
                "SetTextSampleData",
                "BeginFullScreen",
                # bgen gets the argument in/out wrong..
                "AddTextSample",
                "AddTESample",
                "AddHiliteSample",
                "HiliteTextSample",

                "MakeTrackTimeTable", # Uses long * return?
                "MakeMediaTimeTable", # ditto
##                      "VideoMediaGetStallCount", # Undefined in CW Pro 3 library
                # OS8 only:
                'SpriteMediaGetIndImageProperty',       # XXXX Why isn't this in carbon?
                'CheckQuickTimeRegistration',
                'SetMovieAnchorDataRef',
                'GetMovieAnchorDataRef',
                'GetMovieLoadState',
                'OpenADataHandler',
                'MovieMediaGetCurrentMovieProperty',
                'MovieMediaGetCurrentTrackProperty',
                'MovieMediaGetChildMovieDataReference',
                'MovieMediaSetChildMovieDataReference',
                'MovieMediaLoadChildMovieFromDataReference',
                'Media3DGetViewObject',

    # these are ImageCompression blacklists
                "GraphicsExportGetInputPtr",

                # QuickTimeComponents
                # These two need some help: the first returns a point to a databuffer that
                # the second disposes. Generate manually?
                "VDCompressDone",
                "VDReleaseCompressBuffer",
                "QTVideoOutputGetGWorldParameters", # How useful is this?

                # MediaHandlers
                "MediaMakeMediaTimeTable", # just lazy
                "MediaGetSampleDataPointer", # funny output pointer

                # QuickTimeMusic
                "kControllerMinimum",
                # These are artefacts of a macro definition
                "ulen",
                "_ext",
                "x",
                "w1",
                "w2",
                ]

    def makeblacklisttypes(self):
        return [
                # I don't think we want to do these
                "QTSyncTaskPtr",
                # We dont do callbacks yet, so no need for these
                "QTCallBack",
                # Skipped for now, due to laziness
                "TrackEditState",
                "MovieEditState",
                "MatrixRecord",
                "MatrixRecord_ptr",
                "SampleReferencePtr",
                "QTTweener",
                "QTErrorReplacementPtr",
                "QTRestrictionSet",
                "QTUUID",
                "QTUUID_ptr",

                # Routine pointers, not yet.
                "MoviesErrorUPP",
                "MoviePreviewCallOutUPP",
                "MovieDrawingCompleteUPP",
                "QTCallBackUPP",
                "TextMediaUPP",
                "MovieProgressUPP",
                "MovieRgnCoverUPP",
                "MCActionFilterUPP",
                "MCActionFilterWithRefConUPP",
                "GetMovieUPP",
                "ModalFilterUPP",
                "TrackTransferUPP",
                "MoviePrePrerollCompleteUPP",
                "MovieExecuteWiredActionsUPP",
                "QTBandwidthNotificationUPP",
                "DoMCActionUPP",
                "QTNextTaskNeededSoonerCallbackUPP",

                "SampleReference64Ptr", # Don't know what this does, yet
                "QTRuntimeSpriteDescPtr",
                "QTBandwidthReference",
                "QTScheduledBandwidthReference",
                "QTAtomContainer",
                "SpriteWorld",
                "Sprite",

    # these are ImageCompression blacklists
    "ICMDataUPP",
    "ICMFlushUPP",
    "ICMCompletionUPP",
    "ICMProgressUPP",
    "StdPixUPP",
    "QDPixUPP",
    "ICMAlignmentUPP",
    "ICMCursorShieldedUPP",
    "ICMMemoryDisposedUPP",
    "ICMConvertDataFormatUPP",
    "ModalFilterYDUPP",
                "FileFilterUPP",

    "CodecNameSpecListPtr",
    "CodecInfo",
     "ImageSequence",
    "MatrixRecordPtr",
    "ICMDataProcRecordPtr",
    "OpenCPicParams",
    "ICMProgressProcRecordPtr",
    "ICMAlignmentProcRecordPtr",
    "ICMPixelFormatInfoPtr",
    "ImageSequenceDataSource",
    "ConstStrFileNameParam",
    "ImageTranscodeSequence",
    "ImageFieldSequence",
    "Fract",
    "PixMapPtr",
    "GWorldFlags",
    "void_ptr",   # XXX Being lazy, this one is doable.

    # These are from QuickTimeComponents
    "CDataHandlerUPP",
    "CharDataHandlerUPP",
    "CommentHandlerUPP",
    "DataHCompletionUPP",
    "'MovieExportGetDataUPP",
    "MovieExportGetPropertyUPP",
    "PreprocessInstructionHandlerUPP",
    "SGModalFilterUPP",
    "StartDocumentHandlerUPP",
    "StartElementHandlerUPP",
    "VdigIntUPP",
    "SGDataUPP",
    "EndDocumentHandlerUPP",
    "EndElementHandlerUPP",
    "VideoBottles", # Record full of UPPs

    "SCParams",
    "ICMCompletionProcRecordPtr",
    "DataHVolumeList",
    "DigitizerInfo",
    "SGCompressInfo",
    "SeqGrabExtendedFrameInfoPtr",
    "SeqGrabFrameInfoPtr",
    "TCTextOptionsPtr",
    "SGCompressInfo_ptr",
    "SGDeviceList",
    "TextDisplayData",
    "TimeCodeDef",
    "TimeCodeRecord",
    "TweenRecord",
    "VDGamRecPtr",
    "ToneDescription",  # XXXX Just lazy: this one is easy.
    "XMLDoc",
    "UInt64",   # XXXX lazy
    "UInt64_ptr", # XXXX lazy

    # From MediaHandlers
    "ActionsUPP",
    "PrePrerollCompleteUPP",
    "CodecComponentHandle", # Difficult: handle containing list of components.
    "GetMovieCompleteParams", # Immense struct
    "LevelMeterInfoPtr", # Lazy. Also: can be an output parameter!!
    "MediaEQSpectrumBandsRecordPtr", # ditto

    # From QuickTimeMusic
    "MusicMIDISendUPP",
    "MusicOfflineDataUPP",
    "TuneCallBackUPP",
    "TunePlayCallBackUPP",
    "GCPart", # Struct with lots of fields
    "GCPart_ptr",
    "GenericKnobDescription", # Struct with lots of fields
    "KnobDescription",  # Struct with lots of fields
    "InstrumentAboutInfo", # Struct, not too difficult
    "NoteChannel", # XXXX Lazy. Could be opaque, I think
    "NoteRequest", # XXXX Lazy. Not-too-difficult struct
    "SynthesizerConnections", # Struct with lots of fields
    "SynthesizerDescription", # Struct with lots of fields
    "TuneStatus", # Struct with lots of fields

                ]

    def makerepairinstructions(self):
        return [
                ([('FSSpec', '*', 'OutMode')], [('FSSpec_ptr', '*', 'InMode')]),

                # Movie controller creation
                ([('ComponentInstance', 'NewMovieController', 'ReturnMode')],
                 [('MovieController', '*', 'ReturnMode')]),

                # NewMovieFromFile
                ([('short', 'resId', 'OutMode'), ('StringPtr', 'resName', 'InMode')],
                 [('short', 'resId', 'InOutMode'), ('dummyStringPtr', 'resName', 'InMode')]),

                # MCDoAction and more
                ([('void', '*', 'OutMode')], [('mcactionparams', '*', 'InMode')]),

                # SetTimeBaseZero. Does not handle NULLs, unfortunately
                ([('TimeRecord', 'zero', 'OutMode')], [('TimeRecord', 'zero', 'InMode')]),

                # ConvertTime and ConvertTimeScale
                ([('TimeRecord', 'inout', 'OutMode')], [('TimeRecord', 'inout', 'InOutMode')]),
                ([('TimeRecord', 'theTime', 'OutMode')], [('TimeRecord', 'theTime', 'InOutMode')]),

                # AddTime and SubtractTime
                ([('TimeRecord', 'dst', 'OutMode')], [('TimeRecord', 'dst', 'InOutMode')]),

                # Funny definitions
                ([('char_ptr', '*', 'InMode')], [('stringptr', '*', 'InMode')]),
                ([('FSSpecPtr', '*', 'InMode')], [('FSSpec_ptr', '*', 'InMode')]),
                ([('unsigned_char', 'swfVersion', 'OutMode')], [('UInt8', 'swfVersion', 'OutMode')]),

                # It seems MusicMIDIPacket if never flagged with const but always used
                # for sending only. If that ever changes this needs to be fixed.
                ([('MusicMIDIPacket', '*', 'OutMode')], [('MusicMIDIPacket_ptr', '*', 'InMode')]),

                # QTMusic const-less input parameters
                ([('unsigned_long', 'header', 'OutMode')], [('UnsignedLongPtr', 'header', 'InMode')]),
                ]

if __name__ == "__main__":
    main()



import os
from test import test_support

# Skip test if _tkinter wasn't built.
test_support.import_module('_tkinter')

this_dir = os.path.dirname(os.path.abspath(__file__))
lib_tk_test = os.path.abspath(os.path.join(this_dir, os.path.pardir,
    'lib-tk', 'test'))

with test_support.DirsOnSysPath(lib_tk_test):
    import runtktests

# Skip test if tk cannot be initialized.
runtktests.check_tk_availability()

def test_main(enable_gui=False):
    if enable_gui:
        if test_support.use_resources is None:
            test_support.use_resources = ['gui']
        elif 'gui' not in test_support.use_resources:
            test_support.use_resources.append('gui')

    with test_support.DirsOnSysPath(lib_tk_test):
        test_support.run_unittest(
            *runtktests.get_tests(text=False, packages=['test_tkinter']))

if __name__ == '__main__':
    test_main(enable_gui=True)
